@@ -1,6 +1,6 @@
 #!/bin/sh
 set -ev
-VERSION=4.52
+VERSION=5.00
 DST=stunnel-$VERSION-android
 
 # to build Zlib:
@@ -15,11 +15,14 @@ DST=stunnel-$VERSION-android
 # make
 # make install
 
-./configure --build=i686-pc-linux-gnu --host=arm-linux-androideabi --prefix=/data/local --with-ssl=/opt/androideabi/sysroot
+mkdir -p bin/android
+cd bin/android
+../../configure --build=i686-pc-linux-gnu --host=arm-linux-androideabi --prefix=/data/local --with-ssl=/opt/androideabi/sysroot
 make clean
 make
+cd ../..
 mkdir $DST
-cp src/stunnel /opt/androideabi/sysroot/bin/openssl $DST
+cp bin/android/src/stunnel /opt/androideabi/sysroot/bin/openssl $DST
 # arm-linux-androideabi-strip $DST/stunnel $DST/openssl
 arm-linux-androideabi-strip $DST/openssl
 zip -r $DST.zip $DST
