@@ -1,6 +1,6 @@
 /*
  *   stunnel       Universal SSL tunnel
- *   Copyright (C) 1998-2012 Michal Trojnara <Michal.Trojnara@mirt.net>
+ *   Copyright (C) 1998-2014 Michal Trojnara <Michal.Trojnara@mirt.net>
  *
  *   This program is free software; you can redistribute it and/or modify it
  *   under the terms of the GNU General Public License as published by the
@@ -59,6 +59,16 @@
 /* for FormatGuard */
 /* #define __NO_FORMATGUARD_ */
 
+/* additional diagnostic messages */
+/* #define DEBUG_FD_ALLOC */
+
+#define DEBUG_INFO
+#ifdef DEBUG_INFO
+#define NOEXPORT
+#else
+#define NOEXPORT static
+#endif
+
 /**************************************** platform */
 
 #ifdef _WIN32
@@ -161,6 +171,7 @@ typedef int socklen_t;
 #endif
 
 /* must be included before sys/stat.h for Ultrix */
+/* must be included before sys/socket.h for OpenBSD */
 #include <sys/types.h>   /* u_short, u_long */
 /* general headers */
 #include <stdio.h>
@@ -201,7 +212,6 @@ typedef unsigned long u32;
 #define readsocket(s,b,n)           recv((s),(b),(n),0)
 #define writesocket(s,b,n)          send((s),(b),(n),0)
 
-/* #define FD_SETSIZE 4096 */
 /* #define Win32_Winsock */
 #define __USE_W32_SOCKETS
 
@@ -264,7 +274,6 @@ typedef unsigned long u32;
     /* OpenVMS compatibility */
 #ifdef __vms
 #define LIBDIR "__NA__"
-#define PIDFILE "SYS$LOGIN:STUNNEL.PID"
 #ifdef __alpha
 #define HOST "alpha-openvms"
 #else
@@ -277,19 +286,22 @@ typedef unsigned long u32;
 #endif  /* __vms */
 
     /* Unix-specific headers */
-#include <signal.h>      /* signal */
-#include <sys/wait.h>    /* wait */
+#include <signal.h>         /* signal */
+#include <sys/wait.h>       /* wait */
 #ifdef HAVE_SYS_RESOURCE_H
-#include <sys/resource.h> /* getrlimit */
+#include <sys/resource.h>   /* getrlimit */
 #endif
 #ifdef HAVE_UNISTD_H
-#include <unistd.h>      /* getpid, fork, execvp, exit */
+#include <unistd.h>         /* getpid, fork, execvp, exit */
 #endif
 #ifdef HAVE_STROPTS_H
 #include <stropts.h>
 #endif
+#ifdef HAVE_MALLOC_H
+#include <malloc.h>         /* mallopt */
+#endif
 #ifdef HAVE_SYS_SELECT_H
-#include <sys/select.h>  /* for aix */
+#include <sys/select.h>     /* for aix */
 #endif
 
 #if defined(HAVE_POLL) && !defined(BROKEN_POLL)
@@ -315,6 +327,10 @@ typedef unsigned long u32;
 #include <posix/grp.h>
 #endif
 
+#ifdef HAVE_SYS_UIO_H
+#include <sys/uio.h>    /* struct iovec */
+#endif /* HAVE_SYS_UIO_H */
+
 #include <netinet/in.h>  /* struct sockaddr_in */
 #include <sys/socket.h>  /* getpeername */
 #include <arpa/inet.h>   /* inet_ntoa */
@@ -358,8 +374,12 @@ extern char *sys_errlist[];
 
 /* Linux */
 #ifdef __linux__
+#ifndef IP_FREEBIND
+/* kernel headers without IP_FREEBIND definition */
+#define IP_FREEBIND 15
+#endif /* IP_FREEBIND */
 #ifndef IP_TRANSPARENT
-/* old kernel headers without IP_TRANSPARENT definition */
+/* kernel headers without IP_TRANSPARENT definition */
 #define IP_TRANSPARENT 19
 #endif /* IP_TRANSPARENT */
 #ifdef HAVE_LINUX_NETFILTER_IPV4_H
@@ -368,6 +388,9 @@ extern char *sys_errlist[];
 #include <linux/netfilter_ipv4.h>
 #endif /* HAVE_LINUX_NETFILTER_IPV4_H */
 #endif /* __linux__ */
+#ifdef HAVE_SYS_SYSCALL_H
+#include <sys/syscall.h> /* SYS_gettid */
+#endif
 
 #endif /* USE_WIN32 */
 
@@ -405,14 +428,21 @@ extern char *sys_errlist[];
 #endif
 #endif /* HAVE_OSSL_ENGINE_H */
 
+/* non-blocking OCSP API is not available before OpenSSL 0.9.8h */
+#if OPENSSL_VERSION_NUMBER<0x00908080L
+#ifdef HAVE_OSSL_OCSP_H
+#undef HAVE_OSSL_OCSP_H
+#endif /* HAVE_OSSL_OCSP_H */
+#endif /* OpenSSL older than 0.9.8h */
+
 #ifdef HAVE_OSSL_OCSP_H
 #include <openssl/ocsp.h>
 #endif /* HAVE_OSSL_OCSP_H */
 
-#ifdef USE_FIPS
+#ifdef HAVE_OSSL_FIPS_H
 #include <openssl/fips.h>
 #include <openssl/fips_rand.h>
-#endif /* USE_FIPS */
+#endif /* HAVE_OSSL_FIPS_H */
 
 #if OPENSSL_VERSION_NUMBER<0x0090800fL
 #define OPENSSL_NO_ECDH
