@@ -1,6 +1,6 @@
 /*
- *   stunnel       TLS offloading and load-balancing proxy
- *   Copyright (C) 1998-2024 Michal Trojnara <Michal.Trojnara@stunnel.org>
+ *   stunnel       Universal SSL tunnel
+ *   Copyright (C) 1998-2014 Michal Trojnara <Michal.Trojnara@mirt.net>
  *
  *   This program is free software; you can redistribute it and/or modify it
  *   under the terms of the GNU General Public License as published by the
@@ -35,6 +35,7 @@
  *   forward this exception.
  */
 
+#include "common.h"
 #include "prototypes.h"
 
 NOEXPORT int main_unix(int, char*[]);
@@ -53,33 +54,24 @@ int main(int argc, char* argv[]) { /* execution begins here 8-) */
 #ifdef M_MMAP_THRESHOLD
     mallopt(M_MMAP_THRESHOLD, 4096);
 #endif
-    tls_init(); /* initialize thread-local storage */
-    crypto_init(); /* initialize libcrypto */
+    str_init(); /* initialize per-thread string management */
     retval=main_unix(argc, argv);
     main_cleanup();
     return retval;
 }
 
 NOEXPORT int main_unix(int argc, char* argv[]) {
-    int configure_status;
-
 #if !defined(__vms) && !defined(USE_OS2)
     int fd;
 
     fd=open("/dev/null", O_RDWR); /* open /dev/null before chroot */
-    if(fd==INVALID_SOCKET)
+    if(fd<0)
         fatal("Could not open /dev/null");
 #endif
-    main_init();
-    configure_status=main_configure(argc>1 ? argv[1] : NULL,
-        argc>2 ? argv[2] : NULL);
-    switch(configure_status) {
-    case 1: /* error -> exit with 1 to indicate error */
+    main_initialize();
+    if(main_configure(argc>1 ? argv[1] : NULL, argc>2 ? argv[2] : NULL)) {
         close(fd);
         return 1;
-    case 2: /* information printed -> exit with 0 to indicate success */
-        close(fd);
-        return 0;
     }
     if(service_options.next) { /* there are service sections -> daemon mode */
 #if !defined(__vms) && !defined(USE_OS2)
@@ -99,7 +91,6 @@ NOEXPORT int main_unix(int argc, char* argv[]) {
         signal(SIGCHLD, signal_handler); /* handle dead children */
         signal(SIGHUP, signal_handler); /* configuration reload */
         signal(SIGUSR1, signal_handler); /* log reopen */
-        signal(SIGUSR2, signal_handler); /* connections */
         signal(SIGPIPE, SIG_IGN); /* ignore broken pipe */
         if(signal(SIGTERM, SIG_IGN)!=SIG_IGN)
             signal(SIGTERM, signal_handler); /* fatal */
@@ -107,25 +98,9 @@ NOEXPORT int main_unix(int argc, char* argv[]) {
             signal(SIGQUIT, signal_handler); /* fatal */
         if(signal(SIGINT, SIG_IGN)!=SIG_IGN)
             signal(SIGINT, signal_handler); /* fatal */
-#endif
-#ifdef USE_FORK
-        setpgid(0, 0); /* create a new process group if needed */
 #endif
         daemon_loop();
-#ifdef USE_FORK
-        s_log(LOG_NOTICE, "Terminating service processes");
-        signal(SIGCHLD, SIG_IGN);
-        signal(SIGTERM, SIG_IGN);
-        kill(0, SIGTERM); /* kill the whole process group */
-        while(wait(NULL)!=-1)
-            ;
-        s_log(LOG_NOTICE, "Service processes terminated");
-#endif
-#if !defined(__vms) && !defined(USE_OS2)
-        delete_pid();
-#endif /* standard Unix */
     } else { /* inetd mode */
-        CLI *c;
 #if !defined(__vms) && !defined(USE_OS2)
         close(fd);
 #endif /* standard Unix */
@@ -133,13 +108,7 @@ NOEXPORT int main_unix(int argc, char* argv[]) {
         signal(SIGCHLD, SIG_IGN); /* ignore dead children */
         signal(SIGPIPE, SIG_IGN); /* ignore broken pipe */
 #endif
-        set_nonblock(0, 1); /* stdin */
-        set_nonblock(1, 1); /* stdout */
-        c=alloc_client_session(&service_options, 0, 1);
-        tls_alloc(c, ui_tls, NULL);
-        service_up_ref(&service_options);
-        client_main(c);
-        client_free(c);
+        client_main(alloc_client_session(&service_options, 0, 1));
     }
     return 0;
 }
@@ -149,7 +118,7 @@ NOEXPORT void signal_handler(int sig) {
     int saved_errno;
 
     saved_errno=errno;
-    signal_post((uint8_t)sig);
+    signal_post(sig);
     signal(sig, signal_handler);
     errno=saved_errno;
 }
@@ -182,7 +151,6 @@ NOEXPORT int daemonize(int fd) { /* go to background */
         exit(0);
     }
 #endif
-    tls_alloc(NULL, ui_tls, "main"); /* reuse thread-local storage */
 #ifdef HAVE_SETSID
     setsid(); /* ignore the error */
 #endif
@@ -197,70 +165,54 @@ NOEXPORT int create_pid(void) {
         s_log(LOG_DEBUG, "No pid file being created");
         return 0;
     }
+    if(global_options.pidfile[0]!='/') {
+        /* to prevent creating pid file relative to '/' after daemonize() */
+        s_log(LOG_ERR, "Pid file (%s) must be full path name", global_options.pidfile);
+        return 1;
+    }
+    global_options.dpid=(unsigned long)getpid();
 
-    /* silently remove the old pid file */
+    /* silently remove old pid file */
     unlink(global_options.pidfile);
-
-    /* create a new pid file */
     pf=open(global_options.pidfile, O_WRONLY|O_CREAT|O_TRUNC|O_EXCL, 0644);
     if(pf==-1) {
         s_log(LOG_ERR, "Cannot create pid file %s", global_options.pidfile);
         ioerror("create");
         return 1;
     }
-    pid=str_printf("%lu\n", (unsigned long)getpid());
-    if(write(pf, pid, strlen(pid))<(int)strlen(pid)) {
-        s_log(LOG_ERR, "Cannot write pid file %s", global_options.pidfile);
-        ioerror("write");
-        return 1;
-    }
+    pid=str_printf("%lu\n", global_options.dpid);
+    write(pf, pid, strlen(pid));
     str_free(pid);
     close(pf);
     s_log(LOG_DEBUG, "Created pid file %s", global_options.pidfile);
+    atexit(delete_pid);
     return 0;
 }
 
 NOEXPORT void delete_pid(void) {
-    if(global_options.pidfile) {
-        if(unlink(global_options.pidfile)<0)
-            ioerror(global_options.pidfile); /* not critical */
-        else
-            s_log(LOG_DEBUG, "Removed pid file %s", global_options.pidfile);
-    } else {
-        s_log(LOG_DEBUG, "No pid file to remove");
-    }
+    if((unsigned long)getpid()!=global_options.dpid)
+        return; /* current process is not main daemon process */
+    s_log(LOG_DEBUG, "removing pid file %s", global_options.pidfile);
+    if(unlink(global_options.pidfile)<0)
+        ioerror(global_options.pidfile); /* not critical */
 }
 
 #endif /* standard Unix */
 
 /**************************************** options callbacks */
 
-void ui_config_reloaded(void) {
+void ui_new_config(void) {
     /* no action */
 }
 
-#ifdef ICON_IMAGE
-
-ICON_IMAGE load_icon_default(ICON_TYPE icon) {
-    (void)icon; /* squash the unused parameter warning */
-    return (ICON_IMAGE)0;
-}
-
-ICON_IMAGE load_icon_file(const char *file) {
-    (void)file; /* squash the unused parameter warning */
-    return (ICON_IMAGE)0;
-}
-
-#endif
-
 /**************************************** client callbacks */
 
-void ui_new_chain(const unsigned section_number) {
-    (void)section_number; /* squash the unused parameter warning */
+void ui_new_chain(const int section_number) {
+    (void)section_number; /* skip warning about unused parameter */
 }
 
-void ui_clients(const long num) {
-    (void)num; /* squash the unused parameter warning */
+void ui_clients(const int num) {
+    (void)num; /* skip warning about unused parameter */
 }
 
 /**************************************** s_log callbacks */
@@ -271,28 +223,20 @@ void ui_new_log(const char *line) {
 
 /**************************************** ctx callbacks */
 
-int ui_passwd_cb(char *buf, int size, int rwflag, void *userdata) {
-    return PEM_def_callback(buf, size, rwflag, userdata);
+int passwd_cb(char *buf, int size, int rwflag, void *userdata) {
+    (void)buf; /* skip warning about unused parameter */
+    (void)size; /* skip warning about unused parameter */
+    (void)rwflag; /* skip warning about unused parameter */
+    (void)userdata; /* skip warning about unused parameter */
+    return 0; /* not implemented */
 }
 
-#ifndef OPENSSL_NO_ENGINE
-
-int (*ui_get_opener()) (UI *) {
-    return UI_method_get_opener(UI_OpenSSL());
+#ifdef HAVE_OSSL_ENGINE_H
+int pin_cb(UI *ui, UI_STRING *uis) {
+    (void)ui; /* skip warning about unused parameter */
+    (void)uis; /* skip warning about unused parameter */
+    return 0; /* not implemented */
 }
-
-int (*ui_get_writer()) (UI *, UI_STRING *) {
-    return UI_method_get_writer(UI_OpenSSL());
-}
-
-int (*ui_get_reader()) (UI *, UI_STRING *) {
-    return UI_method_get_reader(UI_OpenSSL());
-}
-
-int (*ui_get_closer()) (UI *) {
-    return UI_method_get_closer(UI_OpenSSL());
-}
-
 #endif
 
 /* end of ui_unix.c */
