@@ -1,6 +1,6 @@
 /*
- *   stunnel       TLS offloading and load-balancing proxy
- *   Copyright (C) 1998-2025 Michal Trojnara <Michal.Trojnara@stunnel.org>
+ *   stunnel       Universal SSL tunnel
+ *   Copyright (C) 1998-2014 Michal Trojnara <Michal.Trojnara@mirt.net>
  *
  *   This program is free software; you can redistribute it and/or modify it
  *   under the terms of the GNU General Public License as published by the
@@ -35,12 +35,31 @@
  *   forward this exception.
  */
 
+#include "common.h"
 #include "prototypes.h"
 
 int main(int argc, char *argv[]) {
-    if(stunnel_init())
+    static struct WSAData wsa_state;
+    char *c, stunnel_exe_path[MAX_PATH];
+
+    /* set current working directory and engine path */
+    GetModuleFileName(0, stunnel_exe_path, MAX_PATH);
+    c=strrchr(stunnel_exe_path, '\\'); /* last backslash */
+    if(c) /* found */
+        c[1]='\0'; /* truncate program name */
+#ifndef _WIN32_WCE
+    if(!SetCurrentDirectory(stunnel_exe_path)) {
+        fprintf(stderr, "Cannot set directory to %s", stunnel_exe_path);
         return 1;
-    main_init();
+    }
+#endif
+    _putenv_s("OPENSSL_ENGINES", stunnel_exe_path);
+
+    str_init(); /* initialize per-thread string management */
+    if(WSAStartup(MAKEWORD(1, 1), &wsa_state))
+        return 1;
+    resolver_init();
+    main_initialize();
     if(!main_configure(argc>1 ? argv[1] : NULL, argc>2 ? argv[2] : NULL))
         daemon_loop();
     main_cleanup();
@@ -49,75 +68,69 @@ int main(int argc, char *argv[]) {
 
 /**************************************** options callbacks */
 
-void ui_config_reloaded(void) {
+void ui_new_config(void) {
     /* no action */
 }
 
 ICON_IMAGE load_icon_default(ICON_TYPE type) {
-    (void)type; /* squash the unused parameter warning */
+    (void)type; /* skip warning about unused parameter */
     return NULL;
 }
 
 ICON_IMAGE load_icon_file(const char *name) {
-    (void)name; /* squash the unused parameter warning */
+    (void)name; /* skip warning about unused parameter */
     return NULL;
 }
 
 /**************************************** client callbacks */
 
-void ui_new_chain(const unsigned section_number) {
-    (void)section_number; /* squash the unused parameter warning */
+void ui_new_chain(const int section_number) {
+    (void)section_number; /* skip warning about unused parameter */
 }
 
-void ui_clients(const long num) {
-    (void)num; /* squash the unused parameter warning */
+void ui_clients(const int num) {
+    (void)num; /* skip warning about unused parameter */
 }
 
 /**************************************** s_log callbacks */
 
-void message_box(LPCTSTR text, const UINT type) {
-    MessageBox(NULL, text, TEXT("stunnel"), type);
+void message_box(const LPSTR text, const UINT type) {
+    LPTSTR tstr;
+
+    tstr=str2tstr(text);
+    MessageBox(NULL, tstr, TEXT("stunnel"), type);
+    str_free(tstr);
 }
 
 void ui_new_log(const char *line) {
+#ifdef _WIN32_WCE
+    /* log to Windows CE debug output stream */
     LPTSTR tstr;
 
     tstr=str2tstr(line);
-#ifdef _WIN32_WCE
-    /* log to Windows CE debug output stream */
     RETAILMSG(TRUE, (TEXT("%s\r\n"), tstr));
-#else
-    /* use UTF-16 or native codepage rather than UTF-8 */
-    _putts(tstr);
-    fflush(stdout);
-#endif
     str_free(tstr);
+#else
+    printf("%s\n", line);
+#endif
 }
 
 /**************************************** ctx callbacks */
 
-int ui_passwd_cb(char *buf, int size, int rwflag, void *userdata) {
-    return PEM_def_callback(buf, size, rwflag, userdata);
+int passwd_cb(char *buf, int size, int rwflag, void *userdata) {
+    (void)buf; /* skip warning about unused parameter */
+    (void)size; /* skip warning about unused parameter */
+    (void)rwflag; /* skip warning about unused parameter */
+    (void)userdata; /* skip warning about unused parameter */
+    return 0; /* not implemented */
 }
 
-#if !defined(OPENSSL_NO_ENGINE) || OPENSSL_VERSION_NUMBER>=0x10101000L
-
-int (*ui_get_opener(void)) (UI *) {
-    return UI_method_get_opener(UI_OpenSSL());
+#ifdef HAVE_OSSL_ENGINE_H
+int pin_cb(UI *ui, UI_STRING *uis) {
+    (void)ui; /* skip warning about unused parameter */
+    (void)uis; /* skip warning about unused parameter */
+    return 0; /* not implemented */
 }
-
-int (*ui_get_writer(void)) (UI *, UI_STRING *) {
-    return UI_method_get_writer(UI_OpenSSL());
-}
-
-int (*ui_get_reader(void)) (UI *, UI_STRING *) {
-    return UI_method_get_reader(UI_OpenSSL());
-}
-
-int (*ui_get_closer(void)) (UI *) {
-    return UI_method_get_closer(UI_OpenSSL());
-}
-
-#endif /* !defined(OPENSSL_NO_ENGINE) || OPENSSL_VERSION_NUMBER>=0x10101000L */
+#endif
 
 /* end of ui_win_cli.c */
