@@ -1,24 +1,24 @@
 /*
  *   stunnel       Universal SSL tunnel
- *   Copyright (C) 1998-2011 Michal Trojnara <Michal.Trojnara@mirt.net>
+ *   Copyright (C) 1998-2014 Michal Trojnara <Michal.Trojnara@mirt.net>
  *
  *   This program is free software; you can redistribute it and/or modify it
  *   under the terms of the GNU General Public License as published by the
  *   Free Software Foundation; either version 2 of the License, or (at your
  *   option) any later version.
- * 
+ *
  *   This program is distributed in the hope that it will be useful,
  *   but WITHOUT ANY WARRANTY; without even the implied warranty of
  *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  *   See the GNU General Public License for more details.
- * 
+ *
  *   You should have received a copy of the GNU General Public License along
  *   with this program; if not, see <http://www.gnu.org/licenses>.
- * 
+ *
  *   Linking stunnel statically or dynamically with other modules is making
  *   a combined work based on stunnel. Thus, the terms and conditions of
  *   the GNU General Public License cover the whole combination.
- * 
+ *
  *   In addition, as a special exception, the copyright holder of stunnel
  *   gives you permission to combine stunnel with free software programs or
  *   libraries that are released under the GNU LGPL and with code included
@@ -26,7 +26,7 @@
  *   modified versions of such code, with unchanged license). You may copy
  *   and distribute such a system following the terms of the GNU GPL for
  *   stunnel and the licenses of the other code concerned.
- * 
+ *
  *   Note that people who make modified versions of stunnel are not obligated
  *   to grant this special exception for their modified versions; it is their
  *   choice whether to do so. The GNU General Public License gives permission
@@ -41,23 +41,29 @@
 /**************************************** prototypes */
 
 /* verify initialization */
-static int load_file_lookup(X509_STORE *, char *);
-static int add_dir_lookup(X509_STORE *, char *);
+NOEXPORT int load_file_lookup(X509_STORE *, char *);
+NOEXPORT int add_dir_lookup(X509_STORE *, char *);
 
 /* verify callback */
-static int verify_callback(int, X509_STORE_CTX *);
-static int cert_check(CLI *c, X509_STORE_CTX *, int);
-static int crl_check(CLI *c, X509_STORE_CTX *);
+NOEXPORT int verify_callback(int, X509_STORE_CTX *);
+NOEXPORT int verify_checks(int, X509_STORE_CTX *);
+NOEXPORT int cert_check(X509_STORE_CTX *, int);
+NOEXPORT int crl_check(X509_STORE_CTX *);
 #ifdef HAVE_OSSL_OCSP_H
-static int ocsp_check(CLI *c, X509_STORE_CTX *);
+NOEXPORT int ocsp_check(X509_STORE_CTX *);
+NOEXPORT OCSP_RESPONSE *ocsp_get_response(CLI *, OCSP_REQUEST *);
 #endif
 
 /* utility functions */
-static void log_time(const int, const char *, ASN1_TIME *);
+NOEXPORT void log_time(const int, const char *, ASN1_TIME *);
 
 /**************************************** verify initialization */
 
 int verify_init(SERVICE_OPTIONS *section) {
+    STACK_OF(X509_NAME) *ca_dn;
+    char *ca_name;
+    int i;
+
     if(section->verify_level<0)
         return 0; /* OK - no certificate verification */
 
@@ -81,13 +87,22 @@ int verify_init(SERVICE_OPTIONS *section) {
             sslerror("SSL_CTX_load_verify_locations");
             return 1; /* FAILED */
         }
-        /* list of trusted CAs for the client to choose the right cert */
-        SSL_CTX_set_client_CA_list(section->ctx,
-            SSL_load_client_CA_file(section->ca_file));
-        s_log(LOG_DEBUG, "Loaded verify certificates from %s",
-            section->ca_file);
+        /* revocation store needs CA certificates for CRL validation */
         if(load_file_lookup(section->revocation_store, section->ca_file))
             return 1; /* FAILED */
+        /* trusted CA names sent to clients for client cert selection */
+        if(!section->option.client) { /* only performed on server */
+            s_log(LOG_DEBUG, "Client CA list: %s",
+                section->ca_file);
+            ca_dn=SSL_load_client_CA_file(section->ca_file);
+            for (i=0; i<sk_X509_NAME_num(ca_dn); ++i) {
+                ca_name=X509_NAME_oneline(sk_X509_NAME_value(ca_dn, i),
+                    NULL, 0);
+                s_log(LOG_INFO, "Client CA: %s", ca_name);
+                OPENSSL_free(ca_name);
+            }
+            SSL_CTX_set_client_CA_list(section->ctx, ca_dn);
+        }
     }
 
     if(section->ca_dir) {
@@ -120,7 +135,7 @@ int verify_init(SERVICE_OPTIONS *section) {
     return 0; /* OK */
 }
 
-static int load_file_lookup(X509_STORE *store, char *name) {
+NOEXPORT int load_file_lookup(X509_STORE *store, char *name) {
     X509_LOOKUP *lookup;
 
     lookup=X509_STORE_add_lookup(store, X509_LOOKUP_file());
@@ -137,7 +152,7 @@ static int load_file_lookup(X509_STORE *store, char *name) {
     return 0; /* OK */
 }
 
-static int add_dir_lookup(X509_STORE *store, char *name) {
+NOEXPORT int add_dir_lookup(X509_STORE *store, char *name) {
     X509_LOOKUP *lookup;
 
     lookup=X509_STORE_add_lookup(store, X509_LOOKUP_hash_dir());
@@ -156,97 +171,133 @@ static int add_dir_lookup(X509_STORE *store, char *name) {
 
 /**************************************** verify callback */
 
-static int verify_callback(int preverify_ok, X509_STORE_CTX *callback_ctx) {
+NOEXPORT int verify_callback(int preverify_ok, X509_STORE_CTX *callback_ctx) {
         /* our verify callback function */
     SSL *ssl;
     CLI *c;
-    char *subject_name;
 
     /* retrieve application specific data */
     ssl=X509_STORE_CTX_get_ex_data(callback_ctx,
         SSL_get_ex_data_X509_STORE_CTX_idx());
     c=SSL_get_ex_data(ssl, cli_index);
-
-    /* certificate name for logging */
-    subject_name=X509_NAME_oneline(
-        X509_get_subject_name(callback_ctx->current_cert), NULL, 0);
-
-    s_log(LOG_DEBUG, "Starting certificate verification: depth=%d, %s",
-        callback_ctx->error_depth, subject_name);
-    if(!cert_check(c, callback_ctx, preverify_ok)) {
-        s_log(LOG_WARNING, "Certificate check failed: depth=%d, %s",
-            callback_ctx->error_depth, subject_name);
-        OPENSSL_free(subject_name);
-        return 0; /* reject connection */
+    if(c->opt->verify_level<1) {
+        s_log(LOG_INFO, "Certificate verification disabled");
+        return 1; /* accept */
     }
-    if(!crl_check(c, callback_ctx)) {
-        s_log(LOG_WARNING, "CRL check failed: depth=%d, %s",
-            callback_ctx->error_depth, subject_name);
-        OPENSSL_free(subject_name);
-        return 0; /* reject connection */
+    if(verify_checks(preverify_ok, callback_ctx))
+        return 1; /* accept */
+    if(c->opt->option.client || c->opt->protocol>=0)
+        return 0; /* reject */
+    if(c->opt->redirect_addr.num) { /* pre-resolved addresses */
+        addrlist_dup(&c->connect_addr, &c->opt->redirect_addr);
+        s_log(LOG_INFO, "Redirecting connection");
+        return 1; /* accept */
+    }
+    /* delayed lookup */
+    if(namelist2addrlist(&c->connect_addr,
+            c->opt->redirect_list, DEFAULT_LOOPBACK)) {
+        s_log(LOG_INFO, "Redirecting connection");
+        return 1; /* accept */
+    }
+    return 0; /* reject */
+}
+
+NOEXPORT int verify_checks(int preverify_ok, X509_STORE_CTX *callback_ctx) {
+    X509 *cert;
+    int depth;
+    char *subject;
+
+    cert=X509_STORE_CTX_get_current_cert(callback_ctx);
+    depth=X509_STORE_CTX_get_error_depth(callback_ctx);
+    subject=X509_NAME_oneline(X509_get_subject_name(cert), NULL, 0);
+
+    s_log(LOG_DEBUG, "Starting certificate verification: depth=%d, subject=%s",
+        depth, subject);
+
+    if(!cert_check(callback_ctx, preverify_ok)) {
+        s_log(LOG_WARNING, "Certificate check failed: depth=%d, subject=%s",
+            depth, subject);
+        OPENSSL_free(subject);
+        return 0; /* fail */
+    }
+    if(!crl_check(callback_ctx)) {
+        s_log(LOG_WARNING, "CRL check failed: depth=%d, subject=%s",
+            depth, subject);
+        OPENSSL_free(subject);
+        return 0; /* fail */
     }
 #ifdef HAVE_OSSL_OCSP_H
-    if(c->opt->option.ocsp && !ocsp_check(c, callback_ctx)) {
-        s_log(LOG_WARNING, "OCSP check failed: depth=%d, %s",
-            callback_ctx->error_depth, subject_name);
-        OPENSSL_free(subject_name);
-        return 0; /* reject connection */
+    if(!ocsp_check(callback_ctx)) {
+        s_log(LOG_WARNING, "OCSP check failed: depth=%d, subject=%s",
+            depth, subject);
+        OPENSSL_free(subject);
+        return 0; /* fail */
     }
 #endif /* HAVE_OSSL_OCSP_H */
-    /* errnum=X509_STORE_CTX_get_error(ctx); */
-    s_log(LOG_NOTICE, "Certificate accepted: depth=%d, %s",
-        callback_ctx->error_depth, subject_name);
-    OPENSSL_free(subject_name);
-    return 1; /* accept connection */
+
+    s_log(depth ? LOG_INFO : LOG_NOTICE,
+        "Certificate accepted: depth=%d, subject=%s", depth, subject);
+    OPENSSL_free(subject);
+    return 1; /* success */
 }
 
 /**************************************** certificate checking */
 
-static int cert_check(CLI *c, X509_STORE_CTX *callback_ctx, int preverify_ok) {
+NOEXPORT int cert_check(X509_STORE_CTX *callback_ctx, int preverify_ok) {
+    SSL *ssl;
+    CLI *c;
     X509_OBJECT obj;
 #if OPENSSL_VERSION_NUMBER>=0x0090700fL
     ASN1_BIT_STRING *local_key, *peer_key;
 #endif
+    X509 *cert;
+    int depth;
+
+    ssl=X509_STORE_CTX_get_ex_data(callback_ctx,
+        SSL_get_ex_data_X509_STORE_CTX_idx());
+    c=SSL_get_ex_data(ssl, cli_index);
+    cert=X509_STORE_CTX_get_current_cert(callback_ctx);
+    depth=X509_STORE_CTX_get_error_depth(callback_ctx);
 
-    if(c->opt->verify_level<1) {
-        s_log(LOG_INFO, "CERT: Verification not enabled");
-        return 1; /* accept connection */
-    }
     if(!preverify_ok) {
         /* remote site specified a certificate, but it's not correct */
-        if(c->opt->verify_level>=4 && callback_ctx->error_depth>0) {
+        if(c->opt->verify_level>=4 && depth>0) {
             s_log(LOG_INFO, "CERT: Invalid CA certificate ignored");
-            return 1; /* accept connection */
-        } else {    
+            return 1; /* success */
+        } else {
             s_log(LOG_WARNING, "CERT: Verification error: %s",
-                X509_verify_cert_error_string(callback_ctx->error));
-            return 0; /* reject connection */
+                X509_verify_cert_error_string(
+                    X509_STORE_CTX_get_error(callback_ctx)));
+            return 0; /* fail */
         }
     }
-    if(c->opt->verify_level>=3 && callback_ctx->error_depth==0) {
+    if(c->opt->verify_level>=3 && depth==0) {
         if(X509_STORE_get_by_subject(callback_ctx, X509_LU_X509,
-                X509_get_subject_name(callback_ctx->current_cert), &obj)!=1) {
-            s_log(LOG_WARNING, "CERT: Certificate not found in local repository");
-            return 0; /* reject connection */
+                X509_get_subject_name(cert), &obj)!=1) {
+            s_log(LOG_WARNING,
+                "CERT: Certificate not found in local repository");
+            return 0; /* fail */
         }
 #if OPENSSL_VERSION_NUMBER>=0x0090700fL
-        peer_key=X509_get0_pubkey_bitstr(callback_ctx->current_cert);
+        peer_key=X509_get0_pubkey_bitstr(cert);
         local_key=X509_get0_pubkey_bitstr(obj.data.x509);
         if(!peer_key || !local_key || peer_key->length!=local_key->length ||
                 memcmp(peer_key->data, local_key->data, local_key->length)) {
             s_log(LOG_WARNING, "CERT: Public keys do not match");
-            return 0; /* reject connection */
+            return 0; /* fail */
         }
 #endif
         s_log(LOG_INFO, "CERT: Locally installed certificate matched");
     }
-    return 1; /* accept connection */
+    return 1; /* success */
 }
 
 /**************************************** CRL checking */
 
 /* based on BSD-style licensed code of mod_ssl */
-static int crl_check(CLI *c, X509_STORE_CTX *callback_ctx) {
+NOEXPORT int crl_check(X509_STORE_CTX *callback_ctx) {
+    SSL *ssl;
+    CLI *c;
     X509_STORE_CTX store_ctx;
     X509_OBJECT obj;
     X509_NAME *subject;
@@ -260,7 +311,9 @@ static int crl_check(CLI *c, X509_STORE_CTX *callback_ctx) {
     char *cp;
     ASN1_TIME *last_update=NULL, *next_update=NULL;
 
-    /* determine certificate ingredients in advance */
+    ssl=X509_STORE_CTX_get_ex_data(callback_ctx,
+        SSL_get_ex_data_X509_STORE_CTX_idx());
+    c=SSL_get_ex_data(ssl, cli_index);
     cert=X509_STORE_CTX_get_current_cert(callback_ctx);
     subject=X509_get_subject_name(cert);
     issuer=X509_get_issuer_name(cert);
@@ -290,7 +343,7 @@ static int crl_check(CLI *c, X509_STORE_CTX *callback_ctx) {
             X509_OBJECT_free_contents(&obj);
             if(pubkey)
                 EVP_PKEY_free(pubkey);
-            return 0; /* reject connection */
+            return 0; /* fail */
         }
         if(pubkey)
             EVP_PKEY_free(pubkey);
@@ -301,13 +354,13 @@ static int crl_check(CLI *c, X509_STORE_CTX *callback_ctx) {
             X509_STORE_CTX_set_error(callback_ctx,
                 X509_V_ERR_ERROR_IN_CRL_NEXT_UPDATE_FIELD);
             X509_OBJECT_free_contents(&obj);
-            return 0; /* reject connection */
+            return 0; /* fail */
         }
         if(X509_cmp_current_time(next_update)<0) {
             s_log(LOG_WARNING, "CRL: CRL Expired - revoking all certificates");
             X509_STORE_CTX_set_error(callback_ctx, X509_V_ERR_CRL_HAS_EXPIRED);
             X509_OBJECT_free_contents(&obj);
-            return 0; /* reject connection */
+            return 0; /* fail */
         }
         X509_OBJECT_free_contents(&obj);
     }
@@ -333,12 +386,12 @@ static int crl_check(CLI *c, X509_STORE_CTX *callback_ctx) {
                 OPENSSL_free(cp);
                 X509_STORE_CTX_set_error(callback_ctx, X509_V_ERR_CERT_REVOKED);
                 X509_OBJECT_free_contents(&obj);
-                return 0; /* reject connection */
+                return 0; /* fail */
             }
         }
         X509_OBJECT_free_contents(&obj);
     }
-    return 1; /* accept connection */
+    return 1; /* success */
 }
 
 #ifdef HAVE_OSSL_OCSP_H
@@ -346,12 +399,13 @@ static int crl_check(CLI *c, X509_STORE_CTX *callback_ctx) {
 /**************************************** OCSP checking */
 /* TODO: check OCSP server specified in the certificate */
 
-static int ocsp_check(CLI *c, X509_STORE_CTX *callback_ctx) {
+NOEXPORT int ocsp_check(X509_STORE_CTX *callback_ctx) {
+    SSL *ssl;
+    CLI *c;
     int error, retval=0;
     X509 *cert;
     X509 *issuer=NULL;
     OCSP_CERTID *certID;
-    BIO *bio=NULL;
     OCSP_REQUEST *request=NULL;
     OCSP_RESPONSE *response=NULL;
     OCSP_BASICRESP *basicResponse=NULL;
@@ -359,17 +413,15 @@ static int ocsp_check(CLI *c, X509_STORE_CTX *callback_ctx) {
         *this_update=NULL, *next_update=NULL;
     int status, reason;
 
-    /* connect specified OCSP server (responder) */
-    c->fd=s_socket(c->opt->ocsp_addr.sa.sa_family, SOCK_STREAM, 0,
-        0, "OCSP: socket (auth_user)");
-    if(c->fd<0)
-        return 0; /* reject connection */
-    if(connect_blocking(c, &c->opt->ocsp_addr, addr_len(&c->opt->ocsp_addr)))
-        goto cleanup;
-    s_log(LOG_DEBUG, "OCSP: server connected");
+    ssl=X509_STORE_CTX_get_ex_data(callback_ctx,
+        SSL_get_ex_data_X509_STORE_CTX_idx());
+    c=SSL_get_ex_data(ssl, cli_index);
+
+    if(!c->opt->option.ocsp)
+        return 1; /* success */
 
     /* get current certificate ID */
-    cert=X509_STORE_CTX_get_current_cert(callback_ctx); /* get current cert */
+    cert=X509_STORE_CTX_get_current_cert(callback_ctx);
     if(X509_STORE_CTX_get1_issuer(&issuer, callback_ctx, cert)!=1) {
         sslerror("OCSP: X509_STORE_CTX_get1_issuer");
         goto cleanup;
@@ -393,14 +445,9 @@ static int ocsp_check(CLI *c, X509_STORE_CTX *callback_ctx) {
     OCSP_request_add1_nonce(request, 0, -1);
 
     /* send the request and get a response */
-    /* FIXME: this code won't work with ucontext threading */
-    /* (blocking sockets are used) */
-    bio=BIO_new_fd(c->fd, BIO_NOCLOSE);
-    response=OCSP_sendreq_bio(bio, c->opt->ocsp_path, request);
-    if(!response) {
-        sslerror("OCSP: OCSP_sendreq_bio");
+    response=ocsp_get_response(c, request);
+    if(!response)
         goto cleanup;
-    }
     error=OCSP_response_status(response);
     if(error!=OCSP_RESPONSE_STATUS_SUCCESSFUL) {
         s_log(LOG_WARNING, "OCSP: Responder error: %d: %s",
@@ -447,10 +494,8 @@ static int ocsp_check(CLI *c, X509_STORE_CTX *callback_ctx) {
         log_time(LOG_NOTICE, "OCSP: Revoked at", revoked_at);
         goto cleanup;
     }
-    retval=1; /* accept connection */
+    retval=1; /* success */
 cleanup:
-    if(bio)
-        BIO_free_all(bio);
     if(issuer)
         X509_free(issuer);
     if(request)
@@ -459,14 +504,68 @@ cleanup:
         OCSP_RESPONSE_free(response);
     if(basicResponse)
         OCSP_BASICRESP_free(basicResponse);
-    closesocket(c->fd);
-    c->fd=-1; /* avoid double close on cleanup */
     return retval;
 }
 
+NOEXPORT OCSP_RESPONSE *ocsp_get_response(CLI *c, OCSP_REQUEST *req) {
+    BIO *bio=NULL;
+    OCSP_REQ_CTX *req_ctx=NULL;
+    OCSP_RESPONSE *resp=NULL;
+    int err;
+
+    /* connect specified OCSP server (responder) */
+    c->fd=s_socket(c->opt->ocsp_addr.sa.sa_family, SOCK_STREAM, 0,
+        1, "OCSP: socket (auth_user)");
+    if(c->fd<0)
+        goto cleanup;
+    if(s_connect(c, &c->opt->ocsp_addr, addr_len(&c->opt->ocsp_addr)))
+        goto cleanup;
+    bio=BIO_new_fd(c->fd, BIO_NOCLOSE);
+    if(!bio)
+        goto cleanup;
+    s_log(LOG_DEBUG, "OCSP: server connected");
+
+    /* OCSP protocol communication loop */
+    req_ctx=OCSP_sendreq_new(bio, c->opt->ocsp_path, req, -1);
+    if(!req_ctx) {
+        sslerror("OCSP: OCSP_sendreq_new");
+        goto cleanup;
+    }
+    while(OCSP_sendreq_nbio(&resp, req_ctx)==-1) {
+        s_poll_init(c->fds);
+        s_poll_add(c->fds, c->fd, BIO_should_read(bio), BIO_should_write(bio));
+        err=s_poll_wait(c->fds, c->opt->timeout_busy, 0);
+        if(err==-1)
+            sockerror("OCSP: s_poll_wait");
+        if(err==0)
+            s_log(LOG_INFO, "OCSP: s_poll_wait: TIMEOUTbusy exceeded");
+        if(err<=0)
+            goto cleanup;
+    }
+    /* s_log(LOG_DEBUG, "OCSP: context state: 0x%x", *(int *)req_ctx); */
+    /* http://www.mail-archive.com/openssl-users@openssl.org/msg61691.html */
+    if(!resp) {
+        if(ERR_peek_error())
+            sslerror("OCSP: OCSP_sendreq_nbio");
+        else /* OpenSSL error: OCSP_sendreq_nbio does not use OCSPerr */
+            s_log(LOG_ERR, "OCSP: OCSP_sendreq_nbio: OpenSSL internal error");
+    }
+
+cleanup:
+    if(req_ctx)
+        OCSP_REQ_CTX_free(req_ctx);
+    if(bio)
+        BIO_free_all(bio);
+    if(c->fd>=0) {
+        closesocket(c->fd);
+        c->fd=-1; /* avoid double close on cleanup */
+    }
+    return resp;
+}
+
 #endif /* HAVE_OSSL_OCSP_H */
 
-static void log_time(const int level, const char *txt, ASN1_TIME *t) {
+NOEXPORT void log_time(const int level, const char *txt, ASN1_TIME *t) {
     char *cp;
     BIO *bio;
     int n;
@@ -479,10 +578,6 @@ static void log_time(const int level, const char *txt, ASN1_TIME *t) {
     ASN1_TIME_print(bio, t);
     n=BIO_pending(bio);
     cp=str_alloc(n+1);
-    if(!cp) {
-        BIO_free(bio);
-        return;
-    }
     n=BIO_read(bio, cp, n);
     if(n<0) {
         BIO_free(bio);
